@@ -1,22 +1,25 @@
 package TestApReq::inherit;
-use Apache::Cookie;
-use base qw/Apache::Request Apache::Cookie::Jar/;
+use Apache2::Cookie;
+use base qw/Apache2::Request Apache2::Cookie::Jar/;
 use strict;
 use warnings FATAL => 'all';
+
 use APR;
-use Apache::RequestRec;
-use Apache::RequestIO;
+use Apache2::RequestRec;
+use Apache2::RequestIO;
+use Apache2::Const -compile => qw(OK);
 
 sub handler {
     my $r = shift;
     $r = __PACKAGE__->new($r); # tickles refcnt bug in apreq-1
     die "Wrong package: ", ref $r unless $r->isa('TestApReq::inherit');
     $r->content_type('text/plain');
-    my $j = Apache::Cookie->jar($r->env);
-    my $req = bless { r => $r, j => $j };
+    # look for segfault when $r->isa("Apache2::Request")
+
+    my $req = bless { r => $r };
     $req->printf("method => %s\n", $req->method);
-    $req->printf("cookie => %s\n", $req->cookie("apache")->as_string);
-    return 0;
+    $req->printf("cookie => %s\n", $req->cookies->{"apache"}->as_string);
+    return Apache2::Const::OK;
 }
 
 sub DESTROY { $_[0]->print("DESTROYING ", __PACKAGE__, " object\n") }
