@@ -1,9 +1,10 @@
 /*
-**  Copyright 2004-2006  The Apache Software Foundation
-**
-**  Licensed under the Apache License, Version 2.0 (the "License");
-**  you may not use this file except in compliance with the License.
-**  You may obtain a copy of the License at
+**  Licensed to the Apache Software Foundation (ASF) under one or more
+** contributor license agreements.  See the NOTICE file distributed with
+** this work for additional information regarding copyright ownership.
+** The ASF licenses this file to You under the Apache License, Version 2.0
+** (the "License"); you may not use this file except in compliance with
+** the License.  You may obtain a copy of the License at
 **
 **      http://www.apache.org/licenses/LICENSE-2.0
 **
@@ -152,11 +153,12 @@ void at_check(at_t *t, int is_ok, const char *label, const char *file,
         at_trace(t, format, label, file, line);
 
         if (fmt != NULL) {
+            char *f;
             apr_snprintf(format, sizeof format, " format: %s", fmt);
             at_trace(t, "%s", format);
             memcpy(format, "   left:", 8);
-            at_comment(t, format, vp);
-            memcpy(format, "  right:", 8);
+            f = format + strlen(format);
+            apr_snprintf(f, sizeof format - strlen(format), "\n  right: %s", fmt);
             at_comment(t, format, vp);
         }
     }
